package worlds;

import flash.display.Graphics;
import nme.Lib;
import nme.geom.Point;
import nme.events.Event;
import nme.events.MouseEvent;
import nme.text.TextField;
import com.haxepunk.World;
import com.haxepunk.Entity;
import com.haxepunk.HXP;
import com.haxepunk.masks.Hitbox;
import com.haxepunk.utils.Key;
import com.haxepunk.utils.Input;
import com.haxepunk.graphics.Image;
//import entities.Test;
import worlds.Intro;
import entities.Cursor;
import entities.Tile;
import entities.ShowPlayer;
import entities.Winner;

class Play extends World
{
	public static inline var brdTop: Int = 60;
	public static inline var brdLeft: Int = 60;
	public static inline var brdWidth: Int = 9;
	public static inline var brdHeight: Int = 9;
	public static var tfInfo: TextField;
	public static var showP: Array<ShowPlayer>;
	public static var mouseClick: Bool;
	public static var gaming: Bool;
	public static var board: Array<Array<Int>>;
	public static var boardF: Array<Array<Int>>;
	public static var tiles: Array<Array<Tile>>;
	public static var player: Array<Cursor>;
	public static var playing: Int;
	public static var pointX: Int = 0;
	public static var pointY: Int = 0;
	public static var winImg: Winner;
	private var curChange: Bool;
	private var fonboard: Bool;
	private var fempty: Bool;
	private var fvoid: Bool;
	private var ffull: Bool;
	private var fillMaster: Bool;
	
    public function new()
    {
        super();

		player = new Array();
		player[1] = new Cursor(1);
		player[2] = new Cursor(2);
		showP = new Array();
		showP[1] = new ShowPlayer(1+60,1,1);
		showP[2] = new ShowPlayer(1 + ((brdWidth + 1) * 60) - 200, 1, 2);
		winImg = null;

        board = new Array();
        boardF = new Array();
        tiles = new Array();
        for (ix in 1...(brdWidth + 1)) {
			board[ix] = new Array();
			boardF[ix] = new Array();
			tiles[ix] = new Array();
			for (iy in 1...(brdHeight + 1)) {
				tiles[ix][iy] = new Tile(1 + brdLeft + 60 * (ix - 1), 1 + brdTop + 60 * (iy - 1));
			}
		}

		gaming = false;
		mouseClick = false;
    }
	
	public override function begin() : Void
	{
	  	super.begin();

		tfInfo = new TextField();
		tfInfo.x = 5;
		tfInfo.y = 5;
        tfInfo.width = 200;
		tfInfo.height = 20;
 		tfInfo.textColor = 0xFFFFFF;
        tfInfo.text = "";
        nme.Lib.current.addChild(tfInfo);

		addGraphic(Image.createRect(Main.myWidth, Main.myHeight, Main.myBgColor));
		player[1].hide();
		player[2].hide();
		add(player[1]);
		add(player[2]);
		add(showP[1]);
		add(showP[2]);
		
		startGame();
  	}
	
	public function startGame(): Void
	{
//		removeAll();
		if (winImg != null) { remove(winImg); }

		mouseClick = false;
		
		curChange = false;
		fonboard = false;
		fempty = false;
		fvoid = false;
		ffull = false;
		
		for(ix in 1...(brdWidth + 1)) {
			for (iy in 1...(brdHeight+1)) {
				var cnt: Int = 0;
				if ((ix == 1) || (iy == 1) || (ix == brdWidth) || (iy == brdHeight)) {
					cnt = 3;
				}
				board[ix][iy] = cnt;
				boardF[ix][iy] = 0;
				tiles[ix][iy].setEmpty();
				add(tiles[ix][iy]);
			}
		}

		playing = Std.random(2)+1;
		
		gaming = true;
		checkCursor;
		redrawBoard();
	}

	public function endGame(): Void
	{
		gaming = false;
//		removeAll();
		mouseClick = false;
		player[1].hide();
		player[2].hide();
		showP[1].show(false, 0);
		showP[2].show(false, 0);
	}

    public override function update() : Void
    {
		if (gaming) {
			checkCursor();
			if (Input.mousePressed) {
				mouseClick = true;
			} else if (Input.mouseReleased && mouseClick) {
				mouseClick = false;
				Validate();
			}
		} else {
			if (Input.mousePressed) {
				mouseClick = true;
			} else if (Input.mouseReleased && mouseClick) {
				mouseClick = false;
				HXP.world = new Intro();
			}
		}
  	    super.update();
	}

	public function checkCursor():Void
	{
		var posX: Int;
		var posY: Int;
		var curX: Int;
		var curY: Int;
		
		posX = Math.round(this.mouseX);
		posY = Math.round(this.mouseY);
		curX = 1 + Math.floor((posX - brdLeft + 1) / 60);
		curY = 1 + Math.floor((posY - brdTop + 1) / 60);

		if ((pointX != curX) || (pointY != curY)) {
			pointX = curX;
			pointY = curY;
			curChange = true;
			fonboard = false;
			fempty = false;
			fvoid = false;
			ffull = false;
			if ((pointX < 1) || (pointY < 1) || (pointX > brdWidth) || (pointY > brdHeight)) {
				player[playing].hide();
			} else {
				fonboard = true;
				if ((board[pointX][pointY] == 0) || (board[pointX][pointY] == (3-playing))) {
					fvoid = true;
					player[playing].setVoid();
				} else if ((board[pointX][pointY] == 3) || (board[pointX][pointY] == playing)) {
					fempty = true;
					player[playing].setEmpty();
				} else {
					ffull = true;
					player[playing].setFull();
				}
			}
			player[playing].x = 1 + brdLeft + 60 * (pointX - 1);
			player[playing].y = 1 + brdTop + 60 * (pointY - 1);
		} else {
			curChange = false;
		}

/*		if (playing == 1) { tfInfo.text = "Player : Pink"; }
		else { tfInfo.text = "Player : Cyan"; }
        tfInfo.text = tfInfo.text + "   m=" + Std.string(posX) + "," + Std.string(posY);
        if (fonboard) {
			tfInfo.text = tfInfo.text + " c=" + Std.string(curX) + "," + Std.string(curY);
			if (fvoid) { tfInfo.text = tfInfo.text + " VOI"; }
			if (fempty) { tfInfo.text = tfInfo.text + " EMP"; }
			if (ffull) { tfInfo.text = tfInfo.text + " FUL"; }
		} */
	}
	
	public function Validate(): Void
	{
		var locX: Int;
		var locY: Int;
		var locV: Int;
		var nbC: Int;
		var nbB: Int;
		if ((pointX < 1) || (pointY < 1) || (pointX > brdWidth) || (pointY > brdHeight)) {
			//player[playing].hide;
		} else if (fempty) {
			nbC = 0; nbB = 0;
			board[pointX][pointY] = playing + 3;
			for (locD in 1...5) {
				locX = pointX;
				locY = pointY;
				switch( locD ) {
					case 1:
						locY = pointY-1;
					case 2:
						locX = pointX-1;
					case 3:
						locY = pointY+1;
					case 4:
						locX = pointX+1;
				}
				if ((locX >= 1) && (locX <= brdWidth)
				 && (locY >= 1) && (locY <= brdHeight)) {
					locV = board[locX][locY];
					if (locV == 0) { locV = playing; }
					else if (locV == (3 - playing)) { locV = 3; }
					board[locX][locY] = locV;
					if (locV == (playing + 3)) {
						nbC = nbC + 1;
					}
				} else { nbB = nbB + 1; }
			}
//			if (((nbC==1) && (nbB==1)) || ((nbC>1) && (nbC<4))) { fillBoard(); }
			fillBoard();
			pointX = 0;
			pointY = 0;
			player[playing].hide();
			playing = 3 - playing;
			checkCursor;
			redrawBoard();
		}
	}

	public function fillTile(x: Int, y: Int, val: Int, ref: Int): Int
	{
		var Count: Int;
		
		if ((x < 1) || (y < 1) || (x > brdWidth) || (y > brdHeight)) {
			fillMaster = false;
			return 0;
		} else {
			Count = 0;
			if ((boardF[x][y] <= ref) || (boardF[x][y] == -ref)) {
				boardF[x][y] = val;
				Count = Count + 1
					+ fillTile(x, y - 1, val, ref)
					+ fillTile(x - 1, y, val, ref)
					+ fillTile(x, y + 1, val, ref)
					+ fillTile(x + 1, y, val, ref);
			}
			if (fillMaster) {
				if (board[x][y] == (3 - playing + 3)) { fillMaster = false; }
			}
			return Count;
		}
	}
	public function fillInit(hidechall: Bool): Void {
		for(ix in 1...(brdWidth + 1)) {
			for (iy in 1...(brdHeight + 1)) {
				if (board[ix][iy] == (playing + 3)) { boardF[ix][iy] = 1; }
				else if ((! hidechall) && (board[ix][iy] == ((3-playing) + 3))) { boardF[ix][iy] = 3; }
				else { boardF[ix][iy] = 0; }
			}
		}	
	}
	public function copyFill(): Void
	{
		for(ix in 1...(brdWidth + 1)) {
			for (iy in 1...(brdHeight + 1)) {
				if (boardF[ix][iy] == 2) { board[ix][iy] = playing + 3; }
			}
		}	
	}
	public function fillBoard(): Void
	{
		var nbVS: Int;
		var nbMax: Int;
		var idF: Int;
		var nbS: Array<Int>;
		var fmS: Array<Bool>;
		var locX: Int;
		var locY: Int;
		
		nbS = new Array();
		fmS = new Array();
		
		nbMax = 0;
		nbVS = (brdWidth * brdHeight);
		for(ix in -1...2) {
			for(iy in -1...2) if ((ix != 0) || (iy != 0)) {
				idF = (ix + 2) + 3 * (iy + 1);
				fillInit(true);
				fillTile(pointX, pointY, 2, -1);
				fillTile(pointX - ix, pointY - iy, 4, 1);
				fillMaster = true;
				nbS[idF] = fillTile(pointX + ix, pointY + iy, 2, 1);
				fmS[idF] = fillMaster;
				if (nbS[idF] == nbVS) { nbS[idF] = 0; }
				if (nbS[idF] > nbMax) { nbMax = nbS[idF]; }
			}
		}
		
		for (idF in 1...10) if (idF != 5) {
			if ((nbS[idF] > 0) && ((nbS[idF] < nbMax) || fmS[idF])) {
				fillInit(false);
				fillTile(pointX, pointY, 5, -1);
				locY = Math.floor((idF - 1) / 3);
				locX = idF - (3 * locY);
				locX = locX - 2;
				locY = locY - 1;
				fillTile(pointX - locX, pointY - locY, 4, 3);
//				fillTile(pointX, pointY, 2, -5);
				nbVS = fillTile(pointX + locX, pointY + locY, 2, 1);
				if ( nbVS > 0 ) { copyFill(); }
			}
		}
	}
	
	public function redrawBoard(): Void
	{
		var locX: Int;
		var locY: Int;
		var locV: Int;
		var locP: Int;
		var tlT: Int = 0;
		var tlL: Int = 0;
		var tlB: Int = 0;
		var tlR: Int = 0;
		var cnt1: Int = 0;
		var cnt2: Int = 0;
		var cmp1: Int = 0;
		var cmp2: Int = 0;
		var empty: Int = brdWidth * brdHeight;
		var winP: Int = 0;
		for(ix in 1...(brdWidth + 1)) {
			for (iy in 1...(brdHeight + 1)) {
				locV = board[ix][iy];
				switch( locV ) {
					case 4:
						locP = 1;
						cnt1 = cnt1 + 1;
						empty = empty - 1;
					case 5:
						locP = 2;
						cnt2 = cnt2 + 1;
						empty = empty - 1;
					default:
						locP = 0;
				}
                if (locP>0) {				
					for (locD in 1...5) {
						locX = ix;
						locY = iy;
						switch( locD ) {
							case 1:
								locY = iy-1;
							case 2:
								locX = ix-1;
							case 3:
								locY = iy+1;
							case 4:
								locX = ix+1;
						}
						if ((locX >= 1) && (locX <= brdWidth)
						 && (locY >= 1) && (locY <= brdHeight)) {
							if ((board[locX][locY] - 3) == locP) { locV = 1; }
							else { locV = 0; };
						} else { locV = -1; };
						switch( locD ) {
							case 1:
								tlT = locV;
							case 2:
								tlL = locV;
							case 3:
								tlB = locV;
							case 4:
								tlR = locV;
						}
					}
					tiles[ix][iy].setTile(locP, tlT, tlL, tlB, tlR);
				} else { tiles[ix][iy].setEmpty(); }
			}
		}
		if (cnt1 > cnt2) { cmp1 = 1; cmp2 = -1; }
		else if (cnt1 < cnt2) { cmp1 = -1; cmp2 = 1; }
		else { cmp1 = 0; cmp2 = 0; }
		showP[1].show((playing == 1), cmp1);
		showP[2].show((playing == 2), cmp2);
		if (empty == 0) {
			if (cnt1 > cnt2) { winP = 1; }
			if (cnt1 < cnt2) { winP = 2; }
			winImg = new Winner((((brdWidth + 2) * 60) - 400) / 2, (((brdHeight + 2) * 60) - 200) / 2, winP);
			add(winImg);
			endGame();
		}
	}
}

